#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Trung Le.
# Distributed under the terms of the Modified BSD License.

version_info = (0, 2, 0, 'dev1')
__version__ = '.'.join(map(str, version_info))
